@@ -59,15 +59,6 @@ import six.moves as sm
 # itertools.zip() for Python 2 or zip() for Python 3 - produces an iterator in both cases
 from six.moves import zip
 
-# Python 3 compatibility hack due to no built-in file keyword in Python 3
-# Due to one occurrence of isinstance(<foo>, file) checking to see if something is of file type
-try:
-    # noinspection PyUnboundLocalVariable,PyUnresolvedReferences
-    file
-except NameError:
-    import io
-    file = io.TextIOWrapper
-
 # Detect whether IPython is installed to determine if the built-in "ipy" command should be included
 ipython_available = True
 try:
@@ -84,7 +75,7 @@ try:
 except ImportError:
     pass
 
-__version__ = '0.7.4'
+__version__ = '0.7.5'
 
 # Pyparsing enablePackrat() can greatly speed up parsing, but problems have been seen in Python 3 in the past
 pyparsing.ParserElement.enablePackrat()
@@ -400,32 +391,29 @@ class Cmd(cmd.Cmd):
 
     Line-oriented command interpreters are often useful for test harnesses, internal tools, and rapid prototypes.
     """
-    # Attributes which are NOT dynamically settable at runtime
-    allow_cli_args = True       # Should arguments passed on the command-line be processed as commands?
-    allow_redirection = True    # Should output redirection and pipes be allowed
+    # Attributes used to configure the ParserManager (all are not dynamically settable at runtime)
     blankLinesAllowed = False
-    commentGrammars = pyparsing.Or(
-         [pyparsing.pythonStyleComment, pyparsing.cStyleComment]
-         )
+    case_insensitive = True  # Commands recognized regardless of case
+    commentGrammars = pyparsing.Or([pyparsing.pythonStyleComment, pyparsing.cStyleComment])
     commentInProgress = pyparsing.Literal('/*') + pyparsing.SkipTo(pyparsing.stringEnd ^ '*/')
-
-    default_to_shell = False    # Attempt to run unrecognized commands as shell commands
-    excludeFromHistory = '''run r list l history hi ed edit li eof'''.split()
-    exclude_from_help = ['do_eof']  # Commands to exclude from the help menu
-
-    # make sure your terminators are not in legalChars!
     legalChars = u'!#$%.:?@_-' + pyparsing.alphanums + pyparsing.alphas8bit
     multilineCommands = []  # NOTE: Multiline commands can never be abbreviated, even if abbrev is True
     prefixParser = pyparsing.Empty()
-    redirector = '>'            # for sending output to file
-    reserved_words = []
+    redirector = '>'        # for sending output to file
     shortcuts = {'?': 'help', '!': 'shell', '@': 'load', '@@': '_relative_load'}
-    terminators = [';']
+    terminators = [';']     # make sure your terminators are not in legalChars!
+
+    # Attributes which are NOT dynamically settable at runtime
+    allow_cli_args = True       # Should arguments passed on the command-line be processed as commands?
+    allow_redirection = True    # Should output redirection and pipes be allowed
+    default_to_shell = False    # Attempt to run unrecognized commands as shell commands
+    excludeFromHistory = '''run ru r history histor histo hist his hi h edit edi ed e eof eo eos'''.split()
+    exclude_from_help = ['do_eof', 'do_eos']  # Commands to exclude from the help menu
+    reserved_words = []
 
     # Attributes which ARE dynamically settable at runtime
-    abbrev = True  # Abbreviated commands recognized
+    abbrev = False  # Abbreviated commands recognized
     autorun_on_edit = False  # Should files automatically run after editing (doesn't apply to commands)
-    case_insensitive = True  # Commands recognized regardless of case
     colors = (platform.system() != 'Windows')
     continuation_prompt = '> '
     debug = False
@@ -448,7 +436,6 @@ class Cmd(cmd.Cmd):
     # This starts out as a dictionary but gets converted to an OrderedDict sorted alphabetically by key
     settable = {'abbrev': 'Accept abbreviated commands',
                 'autorun_on_edit': 'Automatically run files after editing',
-                'case_insensitive': 'Upper- and lower-case both OK',
                 'colors': 'Colorized output (*nix only)',
                 'continuation_prompt': 'On 2nd+ line of input',
                 'debug': 'Show full error stack on error',
@@ -511,8 +498,7 @@ class Cmd(cmd.Cmd):
         self._temp_filename = None
 
         # Codes used for exit conditions
-        self._STOP_AND_EXIT = True  # distinguish end of script file from actual exit
-        self._STOP_SCRIPT_NO_EXIT = -999
+        self._STOP_AND_EXIT = True  # cmd convention
 
         self._colorcodes = {'bold': {True: '\x1b[1m', False: '\x1b[22m'},
                             'cyan': {True: '\x1b[36m', False: '\x1b[39m'},
@@ -523,8 +509,8 @@ class Cmd(cmd.Cmd):
                             'underline': {True: '\x1b[4m', False: '\x1b[24m'},
                             'yellow': {True: '\x1b[33m', False: '\x1b[39m'}}
 
-        # Used by load and _relative_load commands
-        self._current_script_dir = None
+        # Used load command to store the current script dir as a LIFO queue to support _relative_load command
+        self._script_dir = []
 
     # -----  Methods related to presenting output to the user -----
 
@@ -833,7 +819,6 @@ class Cmd(cmd.Cmd):
         :return: bool - a flag indicating whether the interpretation of commands should stop
         """
         statement = self.parser_manager.parsed(line)
-        self.lastcmd = statement.parsed.raw
         funcname = self._func_named(statement.parsed.command)
         if not funcname:
             return self._default(statement)
@@ -893,20 +878,23 @@ class Cmd(cmd.Cmd):
     def pseudo_raw_input(self, prompt):
         """copied from cmd's cmdloop; like raw_input, but accounts for changed stdin, stdout"""
 
+        # Deal with the vagaries of readline and ANSI escape codes
+        safe_prompt = self._surround_ansi_escapes(prompt)
+
         if self.use_rawinput:
-            safe_prompt = self._surround_ansi_escapes(prompt)
             try:
                 line = sm.input(safe_prompt)
             except EOFError:
                 line = 'EOF'
         else:
-            self.stdout.write(prompt)
+            self.stdout.write(safe_prompt)
             self.stdout.flush()
             line = self.stdin.readline()
             if not len(line):
                 line = 'EOF'
             else:
                 line = line.rstrip('\r\n')
+
         return line.strip()
 
     def _cmdloop(self):
@@ -933,13 +921,18 @@ class Cmd(cmd.Cmd):
         stop = None
         try:
             while not stop:
-                # NOTE: cmdqueue appears completely unused, but it is defined in cmd.Cmd, so we are leaving it here
                 if self.cmdqueue:
+                    # Run command out of cmdqueue if nonempty (populated by load command or commands at invocation)
                     line = self.cmdqueue.pop(0)
                 else:
+                    # Otherwise, read a command from stdin
                     line = self.pseudo_raw_input(self.prompt)
-                if self.echo and isinstance(self.stdin, file):
+
+                # If echo is on and in the middle of running a script, then echo the line to the output
+                if self.echo and self._current_script_dir is not None:
                     self.stdout.write(line + '\n')
+
+                # Run the command along with all associated pre and post hooks
                 stop = self.onecmd_plus_hooks(line)
         finally:
             if self.use_rawinput and self.completekey:
@@ -948,6 +941,11 @@ class Cmd(cmd.Cmd):
                     readline.set_completer_delims(self.old_delims)
                 except NameError:
                     pass
+
+            # Need to set empty list this way because Python 2 doesn't support the clear() method on lists
+            self.cmdqueue = []
+            self._script_dir = []
+
             return stop
 
     # noinspection PyUnusedLocal
@@ -1024,9 +1022,9 @@ class Cmd(cmd.Cmd):
 
     # noinspection PyUnusedLocal
     def do_eof(self, arg):
-        """Automatically called at end of loading a script or when <Ctrl>-D is pressed."""
+        """Called when <Ctrl>-D is pressed."""
         # End of script should not exit app, but <Ctrl>-D should.
-        return self._STOP_SCRIPT_NO_EXIT
+        return self._STOP_AND_EXIT
 
     def do_quit(self, arg):
         """Exits this application."""
@@ -1595,6 +1593,14 @@ Edited files are run on close if the ``autorun_on_edit`` settable parameter is T
         except Exception as e:
             self.perror('Saving {!r} - {}'.format(fname, e), traceback_war=False)
 
+    @property
+    def _current_script_dir(self):
+        """Accessor to get the current script directory from the _script_dir LIFO queue."""
+        if self._script_dir:
+            return self._script_dir[-1]
+        else:
+            return None
+
     def do__relative_load(self, file_path):
         """Runs commands in script file that is encoded as either ASCII or UTF-8 text.
 
@@ -1612,7 +1618,7 @@ NOTE: This command is intended to only be used within text file scripts.
         """
         # If arg is None or arg is an empty string this is an error
         if not file_path:
-            self.perror('_relative_load command requires a file path:\n', traceback_war=False)
+            self.perror('_relative_load command requires a file path:', traceback_war=False)
             return
 
         file_path = file_path.strip()
@@ -1620,6 +1626,11 @@ NOTE: This command is intended to only be used within text file scripts.
         relative_path = os.path.join(self._current_script_dir or '', file_path)
         self.do_load(relative_path)
 
+    def do_eos(self, _):
+        """Handles cleanup when a script has finished executing."""
+        if self._script_dir:
+            self._script_dir.pop()
+
     def do_load(self, file_path):
         """Runs commands in script file that is encoded as either ASCII or UTF-8 text.
 
@@ -1631,43 +1642,44 @@ Script should contain one command per line, just like command would be typed in
         """
         # If arg is None or arg is an empty string this is an error
         if not file_path:
-            self.perror('load command requires a file path:\n', traceback_war=False)
+            self.perror('load command requires a file path:', traceback_war=False)
             return
 
         expanded_path = os.path.abspath(os.path.expanduser(file_path.strip()))
 
         # Make sure expanded_path points to a file
         if not os.path.isfile(expanded_path):
-            self.perror('{} does not exist or is not a file\n'.format(expanded_path), traceback_war=False)
+            self.perror('{} does not exist or is not a file'.format(expanded_path), traceback_war=False)
             return
 
         # Make sure the file is not empty
         if os.path.getsize(expanded_path) == 0:
-            self.perror('{} is empty\n'.format(expanded_path), traceback_war=False)
+            self.perror('{} is empty'.format(expanded_path), traceback_war=False)
             return
 
         # Make sure the file is ASCII or UTF-8 encoded text
         if not self.is_text_file(expanded_path):
-            self.perror('{} is not an ASCII or UTF-8 encoded text file\n'.format(expanded_path), traceback_war=False)
+            self.perror('{} is not an ASCII or UTF-8 encoded text file'.format(expanded_path), traceback_war=False)
             return
 
         try:
-            target = open(expanded_path)
+            # Specify file encoding in Python 3, but Python 2 doesn't allow that argument to open()
+            if six.PY3:
+                # Add all commands in the script to the command queue
+                with open(expanded_path, encoding='utf-8') as target:
+                    self.cmdqueue.extend(target.read().splitlines())
+            else:
+                # Add all commands in the script to the command queue
+                with open(expanded_path) as target:
+                    self.cmdqueue.extend(target.read().splitlines())
+
+            # Append in an "end of script (eos)" command to cleanup the self._script_dir list
+            self.cmdqueue.append('eos')
         except IOError as e:
             self.perror('Problem accessing script from {}:\n{}'.format(expanded_path, e))
             return
 
-        keepstate = Statekeeper(self, ('stdin', 'use_rawinput', 'prompt',
-                                       'continuation_prompt', '_current_script_dir'))
-        self.stdin = target
-        self.use_rawinput = False
-        self.prompt = self.continuation_prompt = ''
-        self._current_script_dir = os.path.dirname(expanded_path)
-        stop = self._cmdloop()
-        self.stdin.close()
-        keepstate.restore()
-        self.lastcmd = ''
-        return stop and (stop != self._STOP_SCRIPT_NO_EXIT)
+        self._script_dir.append(os.path.dirname(expanded_path))
 
     def do_run(self, arg):
         """run [arg]: re-runs an earlier command
@@ -1732,16 +1744,6 @@ Script should contain one command per line, just like command would be typed in
         runner = unittest.TextTestRunner()
         runner.run(testcase)
 
-    def _run_commands_at_invocation(self, callargs):
-        """Runs commands provided as arguments on the command line when the application is started.
-
-        :param callargs: List[str] - list of strings where each string is a command plus its arguments
-        :return: bool - True implies the entire application should exit
-        """
-        for initial_command in callargs:
-            if self.onecmd_plus_hooks(initial_command + '\n'):
-                return self._STOP_AND_EXIT
-
     def cmdloop(self, intro=None):
         """This is an outer wrapper around _cmdloop() which deals with extra features provided by cmd2.
 
@@ -1753,19 +1755,25 @@ Script should contain one command per line, just like command would be typed in
 
         :param intro: str - if provided this overrides self.intro and serves as the intro banner printed once at start
         """
-        callargs = None
         if self.allow_cli_args:
             parser = optparse.OptionParser()
             parser.add_option('-t', '--test', dest='test',
                               action="store_true",
                               help='Test against transcript(s) in FILE (wildcards OK)')
             (callopts, callargs) = parser.parse_args()
+
+            # If transcript testing was called for, use other arguments as transcript files
             if callopts.test:
                 self._transcript_files = callargs
 
+            # If commands were supplied at invocation, then add them to the command queue
+            if callargs:
+                self.cmdqueue.extend(callargs)
+
         # Always run the preloop first
         self.preloop()
 
+        # If transcript-based regression testing was requested, then do that instead of the main loop
         if self._transcript_files is not None:
             self.run_transcript_tests(self._transcript_files)
         else:
@@ -1777,14 +1785,8 @@ Script should contain one command per line, just like command would be typed in
             if self.intro is not None:
                 self.stdout.write(str(self.intro) + "\n")
 
-            stop = False
-            # If allowed, process any commands present as arguments on the command-line, if allowed
-            if self.allow_cli_args:
-                stop = self._run_commands_at_invocation(callargs)
-
-            # And then call _cmdloop() if there wasn't something in those causing us to quit
-            if not stop:
-                self._cmdloop()
+            # And then call _cmdloop() to enter the main loop
+            self._cmdloop()
 
         # Run the postloop() no matter what
         self.postloop()
@@ -2270,36 +2272,7 @@ class CmdResult(namedtuple_with_two_defaults('CmdResult', ['out', 'err', 'war'])
 if __name__ == '__main__':
     # If run as the main application, simply start a bare-bones cmd2 application with only built-in functionality.
 
-    # Set this to True to include the ipy command if IPython is installed, which supports advanced interactive debugging
-    # of your application via introspection on self.
-    # include_ipy = False
-    #
-    # app = Cmd(use_ipython=include_ipy)
-    # app.cmdloop()
-
-
-    class HelpApp(Cmd):
-        """Class for testing custom help_* methods which override docstring help."""
-
-        def __init__(self, *args, **kwargs):
-            # Need to use this older form of invoking super class constructor to support Python 2.x and Python 3.x
-            Cmd.__init__(self, *args, **kwargs)
-
-        def do_squat(self, arg):
-            """This docstring help will never be shown because the help_squat method overrides it."""
-            pass
-
-        def help_squat(self):
-            self.stdout.write('This command does diddly squat...\n')
-
-        def do_edit(self, arg):
-            """This overrides the edit command and does nothing."""
-            pass
-
-        # This command will be in the "undocumented" section of the help menu
-        def do_undoc(self, arg):
-            pass
-
-
-    app = HelpApp()
+    # Set "use_iptyhon" to True to include the ipy command if IPython is installed, which supports advanced interactive
+    # debugging of your application via introspection on self.
+    app = Cmd(use_ipython=False)
     app.cmdloop()
